<?
function error($error, $refer = "") {
	$error = urlencode($error);
	if(!empty($refer))
		$error .= urlencode("<a href=\"$refer\">Try again</a>\n");
	else
		$error .= urlencode("<a href=\"javascript:history.go(-1);\">Go back</a>\n");
	header("Location: .?page=error&error=$error");
	exit();
}

function check_config() {
	global $config;

	// Check is files are present and readable/executable
	$error = "";
	if(!is_executable($config['ex_sudo']))
		$error .= "sudo is not executable\n";
	if(!is_file($config['ex_rm']))
		$error .= "rm not found\n";
	if(!is_file($config['ex_chmod']))
		$error .= "chmod not found\n";
	if(!is_file($config['ex_tee']))
		$error .= "tee not found\n";
	if(!is_file($config['ex_useradd']))
		$error .= "useradd not found\n";
	if(!is_file($config['ex_usermod']))
		$error .= "usermod not found\n";
	if(!is_file($config['ex_userdel']))
		$error .= "userdel not found\n";
	if(!is_file($config['ex_passwd']))
		$error .= "passwd not found\n";
	if(!is_dir($config['ftproot']))
		$error .= "FTP root directory does not exist\n";
	if(!is_readable($config['ftpconfig']))
		$error .= "FTP server configuration file is missing or not readable\n";
	if(!is_readable($config['ftpchrootlist']))
		$error .= "chroot list is missing or not readable\n";
	if(!is_readable($config['passwd']))
		$error .= "passwd-file is missing or not readable\n";
	if(!is_readable($config['group']))
		$error .= "group-file is missing or not readable\n";
	if(!is_readable($config['shells']))
		$error .= "shells-file is missing or not readable\n";

	if(!empty($error))
		error($error, ".");
	
	// Check if loginshell is in the shells-file
	$shell_in_shells = false;

	$file = file($config['shells']);

	foreach($file as $line) {
		$line = trim($line);
		if($config['loginshell'] == $line) {
			$shell_in_shells = true;
			break;
		}
	}

	if(!$shell_in_shells) $error .= "Loginshell is not in shells-file\n";
	if(!empty($error)) error($error, ".");

	// Check if vsftpd config is correct
	$ftp_listen = false;
	$ftp_chrootlist = false;
	$ftp_chrootenable = true;
	$ftp_chrootlocal = false;
	$ftp_localenable = false;

	$file = file($config['ftpconfig']);
	foreach($file as $line) {
		if(preg_match("/^\s*#/", $line)) continue; // ignore comments
		if(preg_match("/^listen\s*=\s*yes/i", $line)) $ftp_listen = true;
		if(preg_match("/^chroot_list_file\s*=\s*" . str_replace("/", "\\/", $config['ftpchrootlist']) . "/i", $line)) $ftp_chrootlist = true;
		if(preg_match("/^chroot_list_enable\s*=\s*yes/i", $line)) $ftp_chrootenable = false;
		if(preg_match("/^chroot_local_users\s*=\s*yes/i", $line)) $ftp_chrootlocal = true;
		if(preg_match("/^local_enable\s*=\s*yes/i", $line)) $ftp_localenable = true;
	}
	$file = null;

	if(!$ftp_listen) $error .= "FTP config: listen must be YES\n";
	if(!$ftp_chrootlist) $error .= "FTP config: chroot_list_file must point to " . $config['ftpchrootlist'] . "\n";
	if(!$ftp_chrootenable) $error .= "FTP config: chroot_list_enable setting must be NO\n";
	if(!$ftp_chrootenable) $error .= "FTP config: chroot_local_users setting must be YES\n";
	if(!$ftp_localenable) $error .= "FTP config: local_enable setting must be YES\n";

	if(!empty($error)) error($error, ".");

	// Check if ftp group exists
	$ftp_group_exists = false;

	$file = file($config['group']);
	foreach($file as $line) {
		list($gname, $gpass, $gid, $gusers) = explode(":", $line);
		if($gname == "ftp") {
			$ftp_group_exists = true;
			break;
		}
	}

	if(!$ftp_group_exists) $error .= "ftp group does not exists (execute: groupadd ftp)\n";

	if(!empty($error)) error($error, ".");

	// Configuration is OK
	$_SESSION['config_ok'] = true;
}

function validate_name($name) {
	if(preg_match("/^[a-zA-Z0-9][a-zA-Z0-9_-]*/", $name))
		return true;
	else
		return false;
}

function list_users() {
	global $config;

	// List all chrooted users
	$file = file($config['ftpchrootlist']);

	$chroot_users = array();

	foreach($file as $line) {
		$line = trim($line);
		$chroot_users[] = $line;
	}

	// List all ftp users in group-file
	$file = file($config['group']);
	foreach($file as $line) {
		$line = trim($line);
		list($gname, $gpass, $gid, $gusers) = explode(":", $line);
		if($gname == "ftp") {
			$ftp_group_id = $gid;
			$ftp_group_users = explode(",", $gusers);
			break;
		}
	}

	// List all ftp users in passwd-file
	$file = file($config['passwd']);

	$users = array();
	
	foreach($file as $line) {
		list($uname, $upass, $uid, $ugid, $udesc, $uhome, $ushell) = explode(":", $line);
		
		if(($ugid == $ftp_group_id) || in_array($uname, $ftp_group_users)) {
			$c = count($users);
			$users[$c]["name"] = $uname;
			$users[$c]["id"] = $uid;
			$users[$c]["desc"] = $udesc;
			$users[$c]["home"] = $uhome;
			$users[$c]["chroot"] = (in_array($uname, $chroot_users)) ? true : false;
		}
	}

	return $users;
}

function chmod_home($home) {
	global $config;

	exec($config['ex_sudo'] . " " . $config['ex_chmod'] . " 0775 " . $home, $out, $ret);
	return $ret;
}

function useradd($name, $desc, $home) {
	global $config;

	exec($config['ex_sudo'] . " " . $config['ex_useradd'] . " -c \"$desc\" -m -d $home -s " . $config['loginshell'] . " -g ftp $name", $out, $ret);
	return $ret;
}

function usermod($name, $desc) {
	global $config;

	exec($config['ex_sudo'] . " " . $config['ex_usermod'] . " -c \"$desc\" $name", $out, $ret);
	return $ret;
}

function userdel($name) {
	global $config;

	exec($config['ex_sudo'] . " " . $config['ex_userdel'] . " $name", $out, $ret);
	return $ret;
}

function removedata($folder) {
	global $config;

	exec($config['ex_sudo'] . " " . $config['ex_rm'] . " -rf $folder", $out, $ret);
	return $ret;
}

function passwd($name, $pass) {
	global $config;

	$l = popen($config['ex_sudo'] . " " . $config['ex_passwd'] . " $name", "w");
	sleep(1);
	fputs($l, $pass);
	sleep(1);
	fputs($l, $pass);
	pclose($l);
}

function set_chroot($name) {
	global $config;

	exec("echo $name | " . $config['ex_sudo'] . " " . $config['ex_tee'] . " -a " . $config['ftpchrootlist'], $out, $ret);
	return $ret;
}

function unset_chroot($name) {
	global $config;

	$data = file_get_contents($config['ftpchrootlist']);
	$data = preg_replace("/^$name\n/m", "", $data);
	$data = preg_replace("/^\n/m", "", $data);
	exec($config['ex_sudo'] . " " . $config['ex_tee'] . " " . $config['ftpchrootlist'] . " <<EOF\n$data\nEOF\n", $out, $ret);
	return $ret;
}

function generate_password() {
	$totalChar = 8;
	$salt = "abcdefghijklmnpqrstuvwxyzABCDEFGHIJKLMNPQRSTUVWXYZ123456789";
	srand((double)microtime() * 1000000);
	for($i = 0; $i < $totalChar; $i++)
		$password .= substr($salt, rand() % strlen($salt), 1);
	return $password;
}
?>
